Sequential Tree Sampler (STS)
=============================

The sequential tree sampler implements a prototype of online phylogenetics, updating a posterior distribution generated by MrBayes with new sequences.
The algorithm has been described and its performance evaluated in [a manuscript][journal]. Also available as a [preprint][preprint].
The scripts used to generate the figures can be found [here](https://github.com/OnlinePhylo/sts-online-experiments).

## Dependencies

* [smctc][smctc] - included as git submodule (`git submodule update --init`)
* [lcfit][lcfit] - included as git submodule (`git submodule update --init`)
* [Bio++ version 2.2.0][bpp] `core`, `seq`, and `phyl` modules. Note that debian & ubuntu up to 16.04 include v2.1.0 which is too old. Bio++ should be installed from source using the `bpp-setup.sh` script on these systems. Alternatively, the source code of version 2.3.0 for each module can be dowloaded from [github](https://github.com/BioPP) in the _releases_ section
* [cmake][cmake]
* [gsl][gsl] version 1 or 2
* [nlopt][nlopt]
* [boost][boost]
* [beagle][beagle] version 2.1 (Optional but recommended)
* [google test][gtest] this is libgtest on debian/ubuntu (Optional)

## Compiling

1. Install dependencies
2. run `make`

Binaries will be build in `_build/release`

## Adding taxa to an existing posterior

The tool `sts-online` adds taxa to an existing posterior tree sample.
`sts-online` operates on a fasta file and tree file in nexus format.
The fasta file must contain an alignment with a superset of the taxa in the tree file.

DNA substitution models: Jukes-Cantor (JC69), generalised time reversible (GTR), Hasegawa Kishino Yano (HKY), Kimura (K80).
Protein substitution models: JTT, WAG, LG.

### Example invocation with JC69

    sts-online -b 250 -p 2 --proposal-method lcfit 10taxon-01.fasta 10tax_trim_t1.t 10tax_trim_t1.sts.json

In this example, we use an alignment containing 10 sequences and a posterior sample of trees generated by MrBayes with an alignment that does not contain the sequence labeled `t1`.
`sts-online` ignores the first 250 trees from `10tax_trim.t1.t` and uses a particle factor of 2. The `10tax_trim_t1.sts.json` file will contain the updated trees.

#### Example invocation with GTR

    sts-online -b 250 -p 2 --proposal-method lcfit 10taxon-01.fasta 10tax_trim_t1.t 10tax_trim_t1.sts.json -P 10tax_trim_t1.p -M GTR -o 10tax

In this example, we use again an alignment containing 10 sequences and a posterior sample of trees generated by MrBayes under the GTR model with an alignment that does not contain the sequence labeled `t1`.
`sts-online` ignores the first 250 trees from `10tax_trim_t1.t` and parameters from `10tax_trim_t1.p` and uses a particle factor of 2. The `10tax_trim_t1.sts.json` file will contain the updated trees and parameters in the json format. Usin the option `-o`, two additional files `10tax.log` and `10tax.trees` containing parameters (csv file) and trees (nexus file) will also be created.


[journal]: http://dx.doi.org/10.1093/sysbio/syx090
[preprint]: http://dx.doi.org/10.1101/145219
[smctc]: http://www2.warwick.ac.uk/fac/sci/statistics/staff/academic-research/johansen/smctc/
[lcfit]: http://github.com/matsengrp/lcfit/
[beagle]: https://code.google.com/p/beagle-lib/
[bpp]: http://biopp.univ-montp2.fr/
[cmake]: http://www.cmake.org/
[gsl]: https://www.gnu.org/software/gsl/
[nlopt]: http://ab-initio.mit.edu/wiki/index.php/NLopt
[boost]: https://www.boost.org
[gtest]: https://github.com/google/googletest

